////////////////////////////////////////////////////////////////////////
// Janne Kataja 13.6.2007
// Part of Gazillion browser
// (c) Copyright 2007 University of Helsinki
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Global variables
////////////////////////////////////////////////////////////////////////

// Segment DOM id:s and bounding boxes for text segments to be tracked
// {
// 	segmentId : DOM ID of DIV segment which contains this element
//	x1 : coordinates from beginning of client viewport
//	x2 : 
//	y1 : 
//	y2 : 
// }
var trackSegments;

// Word SPAN id:s and Bounding boxes for words to be tracked
// {
// 	spanId : DOM ID of SPAN segment which contains this element
//	word : text contents of this word element
//	x1 : coordinates from beginning of client viewport
//	x2 : 
//	y1 : 
//	y2 : 
// }
var trackWords; 

////////////////////////////////////////////////////////////////////////
// Helper functions for coordinates and  window-to-viewport coordinates
// conversion
////////////////////////////////////////////////////////////////////////

// Object is visible in the current client viewport
function inView(that)
{
	var viewportHeight = document.documentElement.clientHeight;
	var objectHeight = that.offsetHeight;
	var offsetTop = pageOffsetTop(that);
	var adjustedHeight = offsetTop - document.documentElement.scrollTop;
	return ((adjustedHeight + objectHeight) > 0 
		&& (adjustedHeight < viewportHeight));
}

// Position of browser window on screen
// TODO Tested only on IE
function getWindowPosition()
{
	var screen = {
		y1 : window.screenTop,
		y2 : window.screenTop + document.documentElement.clientHeight,
		x1 : window.screenLeft,
		x2 : window.screenLeft + document.documentElement.clientWidth
	}
	return screen;
}

// Test for x,y screen coordinates are inside browser viewport
function insideWindow(x,y) 
{
	var vp = getWindowPosition();
	return ((x >= vp.x1 && x <= vp.x2) 
		&& (y >= vp.y1 && y <= vp.y2));
}

// Return screen coordinates adjusted to browser viewport coordinate
// TODO Test this part
function windowToViewport(x,y) 
{
	var wp = getWindowPosition();
	// TODO Add support for horizontal scroll
	var viewportCoordinates = {
		x : x - wp.x1,
		y : y - wp.y1 + document.documentElement.scrollTop
	}
	return viewportCoordinates;
}


// Objects distance to page offset left
function pageOffsetLeft(that) 
{
	var ol = that.offsetLeft;
	while ((that = that.offsetParent) != null) { 
		ol += that.offsetLeft; 
	}
	return ol;
}

// Objects distance to page offset top
function pageOffsetTop(that) 
{
	var ol = that.offsetTop;
	while ((that = that.offsetParent) != null) { 
		ol += that.offsetTop; 
	}
	return ol;
}

////////////////////////////////////////////////////////////////////////
// Populate trackWords and trackSegments structures which contain
// document segments and words, and their respective coordinates
////////////////////////////////////////////////////////////////////////

// Count parts which match tag name and regex
function countTag(tag, regex) 
{
	var tags = document.getElementsByTagName(tag);

	if (tags == undefined || tags.length == 0)
		return 0;

	var count = 0;
	for (var i=0 ; i<tags.length ; i++) {
		if (tags[i].id.match(regex)) {
			++count;
		}
	}
	return count;
}


// Populate trackWords structure
function populateTrackWords() 
{
	trackWords = new Array(countTag('span',/^word:(\d+)$/));
	var spans = document.getElementsByTagName('span');
	var n = 0; 
	// Increase bounding box by this many pixels
	// FIXME Values only for IE with default font configurations
	var growX = 3;
	var growY = 3;
	for (var i=0 ; i<spans.length ; i++) {
		if (!spans[i].id.match(/^word:(\d+)$/)) {
			continue;
		}
		// .innerText supported in IE
		// .textContent supported in Firefox
		trackWords[n] = { 
			word : (navigator.userAgent.match(/MSIE/) 
				? spans[i].innerText 
				: spans[i].textContent),
			spanId : spans[i].id,
			x1 : pageOffsetLeft(spans[i]) - growX,
			x2 : pageOffsetLeft(spans[i]) + spans[i].offsetWidth + growX,
			y1 : pageOffsetTop(spans[i]) - growY,
			y2 : pageOffsetTop(spans[i]) + spans[i].offsetHeight + growY
		}
		n++;
	}
}

// Populate trackSegments structure
function populateTrackSegments() 
{
	trackSegments = new Array(countTag('div',/^segment:(\d+)$/));
	var segments = document.getElementsByTagName('div');
	var n = 0; 
	for (var i=0 ; i<segments.length ; i++) {
		if (!segments[i].id.match(/^segment:(\d+)$/)) {
			continue;
		}
		trackSegments[n] = { 
			segmentId : segments[i].id,
			x1 : pageOffsetLeft(segments[i]),
			x2 : pageOffsetLeft(segments[i]) + segments[i].offsetWidth,
			y1 : pageOffsetTop(segments[i]),
			y2 : pageOffsetTop(segments[i]) + segments[i].offsetHeight
		}
		n++;
	}
}

////////////////////////////////////////////////////////////////////////
// Functions to find which word and segment under window x,y  coords
////////////////////////////////////////////////////////////////////////

// Find which segment is at viewport x,y coordinates
// Returns DIV segment Id
function getViewportSegmentId(x,y)
{
	// FIXME First candidate for of optimization
	for (var i=0 ; i<trackSegments.length ; i++) {
		if ((y >= trackSegments[i].y1 && y <= trackSegments[i].y2)
		&&  (x >= trackSegments[i].x1 && x <= trackSegments[i].x2)) {
			return trackSegments[i].segmentId;
		}
	}
	return "";
}

// Find which word is at viewport x,y coordinates
// Returns SPAN word Id
function getViewportWordId(x,y)
{
	// FIXME First candidate for of optimization
	for (var i=0 ; i<trackWords.length ; i++) {
		if ((y >= trackWords[i].y1 && y <= trackWords[i].y2)
		&&  (x >= trackWords[i].x1 && x <= trackWords[i].x2)) {
			return trackWords[i].spanId;
		}
	}
	return "";
}


// Find which word is at window x,y coordinates
// Returns DIV segment Id
function getWindowSegmentId(x,y)
{
	var vp = windowToViewport(x,y);
	return getViewportSegmentId(vp.x, vp.y);
}

// Find which word is at window x,y coordinates
// Returns SPAN word Id
function getWindowWordId(x,y)
{
	var vp = windowToViewport(x,y);
	return getViewportWordId(vp.x, vp.y);
}

// Find which word is at window x,y coordinates
// Returns text
function getWindowWord(x,y)
{
	var wordId = getWindowWordId(x,y);
	return getWord(wordId);
}

// Return word content for given wordId
function getWord(spanId)
{
	if (spanId == undefined || spanId == "") {
		return "";
	}
	// .innerText supported in IE
	// .textContent supported in Firefox
	return (navigator.userAgent.match(/MSIE/) 
		? document.getElementById(""+spanId).innerText 
		: document.getElementById(""+spanId).textContent);
}

////////////////////////////////////////////////////////////////////////
// Debug functions
////////////////////////////////////////////////////////////////////////

// Debug function for viewport check
function inViewDebug() {
	var str = "";
	for (var i=0 ; i<trackWords.length ; i++) {
		 str += trackWords[i].word 
			+ "," + inView(document.getElementById(trackWords[i].spanId)) + "\n";
	}
	for (var i=0 ; i<trackSegments.length ; i++) {
		 str +=  trackSegments[i].segmentId
			+ "," + inView(document.getElementById(trackSegments[i].segmentId)) + "\n";
	}
	alert(str);
}


// Debug trackWords structure
function populateTrackWordsDebug() 
{
	var str = "";
	for (var i=0 ; i<trackWords.length ; i++) {
		 str += trackWords[i].spanId
			+ "," + trackWords[i].word 
			+ "," + trackWords[i].x1 
			+ "," + trackWords[i].x2
			+ "," + trackWords[i].y1 
			+ "," + trackWords[i].y2 + "\n";
	}
	alert(str);
}

// Debug trackSegments structure
function populateTrackSegmentsDebug() 
{
	var str = "";
	for (var i=0 ; i<trackSegments.length ; i++) {
		 str += trackSegments[i].segmentId
			+ "," + trackSegments[i].x1 
			+ "," + trackSegments[i].x2
			+ "," + trackSegments[i].y1 
			+ "," + trackSegments[i].y2 + "\n";
	}
	alert(str);
}

// Highlight relevant words
function highlightWord(spanId)
{
	if (spanId == undefined || spanId == "") {
		return "";
	}
	document.getElementById(spanId).style.backgroundColor = "#EEEE66"; 
}

// Reset word highlights
function resetHighlights()
{
	for (var i=0 ; i<trackWords.length ; i++) {
		document.getElementById(trackWords[i].spanId).style.backgroundColor = "#FFFFFF"; 
	}
}

////////////////////////////////////////////////////////////////////////
// Search results
////////////////////////////////////////////////////////////////////////

// searchResult array is populated on search page
var searchResult = new Array();

// Jump to result, updating selected document
function showResult(resultNumber)
{
	location = searchResult[resultNumber].page;
}

function resultCount()
{
	return searchResult.length;
}

function resultPage(resultNumber)
{
	return searchResult[resultNumber].page;
}

function resultTitle(resultNumber)
{
	return searchResult[resultNumber].title;
}

function resultDescription(resultNumber)
{
	return searchResult[resultNumber].description;
}

////////////////////////////////////////////////////////////////////////
// Event callbacks
////////////////////////////////////////////////////////////////////////

// Executed on page load and reload
function populateTrack() 
{
	populateTrackWords();
	populateTrackSegments();
	// alert(...)'s populated structures
	//populateTrackWordsDebug();
	//populateTrackSegmentsDebug();
}

window.onload = populateTrack;
window.onresize = populateTrack; 
